default_config = {
    'img_size': 224,
    'patch_size': 16,
    'in_chans': 3,
    'num_classes': 1000,
    'embed_dim': 768,
    'depth': 12,
    'num_heads': 12,
    'mlp_ratio': 4.0,
    'lora': {
        'r': 4,
        'alpha': 1.0,
        'dropout': 0.1,
    },
} 